from _typeshed import Incomplete

debug: bool
wappstoUrl: Incomplete

def start_session(base_url, username, password): ...
def create_network(session, base_url, product: Incomplete | None = ..., test_mode: bool = ..., reset_manufacturer: bool = ..., dry_run: bool = ...): ...
def claim_network(session, base_url, network_uuid, dry_run: bool = ...): ...
def get_network(session, base_url, network_uuid): ...
def create_certificaties_files(location, creator, dry_run) -> None: ...
def main() -> None: ...
